@@ -1,50 +1,55 @@
 package GD::SecurityImage;
 use strict;
-use vars qw[$VERSION];
-use GD;
+use vars qw[@ISA $VERSION];
+use GD::SecurityImage::Styles;
 
-# See the create() method for these constants
-use constant LOW_LEFT_X  => 0;
-use constant LOW_LEFT_Y  => 1;
-use constant LOW_RIGHT_X => 2;
-use constant UP_LEFT_Y   => 7;
+@ISA     = qw(GD::SecurityImage::Styles);
+$VERSION = "1.2";
 
-$VERSION = "1.1";
+sub import {
+   # load the drawing interface
+   my $class = shift;
+   my %opt   = scalar(@_) % 2 ? () : (@_);
+   if ($opt{use_magick}) {
+      require GD::SecurityImage::Magick;
+      push @ISA, qw(GD::SecurityImage::Magick);
+   } else {
+      require GD::SecurityImage::GD;
+      push @ISA, qw(GD::SecurityImage::GD);
+   }
+}
 
 sub new {
    my $class = shift;
    my %opt   = scalar @_ % 2 ? () : (@_);
-   my $self  = {};
+   my $self  = {
+      IS_MAGICK       => defined($Image::Magick::VERSION) ? 1 : 0,
+      MAGICK          => {}, # Image::Magick configuration options
+      _RANDOM_NUMBER_ => '',
+      _RNDMAX_        => 6, # maximum number of characters in a random string.
+   };
    bless $self, $class;
    my %options = (
-               width      => $opt{width}    || 80,
-               height     => $opt{height}   || 30,
-               ptsize     => $opt{ptsize}   || 20,
-               lines      => $opt{lines}    || 10,
-               rndmax     => $opt{rndmax}   || 6,
-               rnd_data   => $opt{rnd_data} || [0..9],
-               font       => $opt{font}     || '',
-               gd_font    => $self->gdf($opt{gd_font}) || GD::Font->Giant,
-               bgcolor    => $opt{bgcolor}  || [255, 255, 255],
-               send_ctobg => $opt{send_ctobg},
+               width      => $opt{width}               || 80,
+               height     => $opt{height}              || 30,
+               ptsize     => $opt{ptsize}              || 20,
+               lines      => $opt{lines}               || 10,
+               rndmax     => $opt{rndmax}              || $self->{_RNDMAX_},
+               rnd_data   => $opt{rnd_data}            || [0..9],
+               font       => $opt{font}                || '',
+               gd_font    => $self->gdf($opt{gd_font}) || '',
+               bgcolor    => $opt{bgcolor}             || [255, 255, 255],
+               send_ctobg => $opt{send_ctobg}          || 0,
    );
    $self->{$_}    = $options{$_} foreach keys %options;
-   $self->{image} = GD::Image->new($self->{width}, $self->{height});
-   $self->{image}->colorAllocate(@{ $self->{bgcolor} }); # set background color
+   $self->init;
    return $self;
 }
 
 sub gdf {
    my $self = shift;
-   my $font = shift || return;
-      $font = lc $font;
-   # GD' s standard fonts
-   my %f = map { lc $_ => $_ } qw[ Small Large MediumBold Tiny Giant ];
-   if (exists $f{$font}) {
-      $font = $f{$font};
-      return GD::Font->$font();
-   }
-   return;
+   return if $self->{IS_MAGICK};
+   return $self->gdfx(@_);
 }
 
 sub random_str {
@@ -55,7 +60,7 @@ sub random_str {
 sub random {
    my $self = shift;
    my $user = shift;
-   if($user and length($user) >= 6) {
+   if($user and length($user) >= $self->{_RNDMAX_}) {
       $self->{_RANDOM_NUMBER_} = $user;
    } else {
       my @keys = @{ $self->{rnd_data} };
@@ -73,118 +78,37 @@ sub create {
    my $style  = shift || 'default'; # default or rect or box
    my $col1   = shift; # text color
    my $col2   = shift; # line/box color
+      $col1 = [ 0, 0, 0] if(not $col1 || not ref $col1 || ref $col1 ne 'ARRAY' || $#{$col1} != 2);
+      $col2 = [ 0, 0, 0] if(not $col2 || not ref $col2 || ref $col2 ne 'ARRAY' || $#{$col2} != 2);
    my %color  = (
-        text  => $self->{image}->colorAllocate($col1 ? @{$col1} : (  0,   0,   0)),
-        lines => $self->{image}->colorAllocate($col2 ? @{$col2} : (200, 200, 200)),
+        text  => $self->{IS_MAGICK} ? $col1 : $self->{image}->colorAllocate(@{$col1}),
+        lines => $self->{IS_MAGICK} ? $col2 : $self->{image}->colorAllocate(@{$col2}),
    );
 
+   if ($method eq 'normal' and not $self->{gd_font}) {
+      $self->{gd_font} = GD::Font->Giant;
+   }
+
    $style = $self->can('style_'.$style) ? 'style_'.$style : 'style_default';
    $self->$style(%color) unless $self->{send_ctobg};
-
-   my $key = $self->{_RANDOM_NUMBER_}; # random string
-
-   if ($method eq 'ttf') {
-      my $methTTF = $GD::VERSION >= 1.31 ? 'stringFT' : 'stringTTF';
-      # don' t draw. we just need info...
-      my @box = GD::Image->$methTTF($color{text},$self->{font},$self->{ptsize},0,0,0,$key)
-                # or die "I can not get the boundary list: $@"
-                # I think that libgd also has some problems 
-                # with paths that have spaces in it.
-                ; 
-      my $x = ($self->{width}  - ($box[LOW_RIGHT_X] - $box[LOW_LEFT_X])) / 2;
-      my $y = ($self->{height} - ($box[UP_LEFT_Y]   - $box[LOW_LEFT_Y])) / 2;
-      $self->{image}->$methTTF($color{text}, $self->{font}, $self->{ptsize}, 0, $x, $y, $key);
-   } else {
-      my $sw = $self->{gd_font}->width * length($key);
-      my $sh = $self->{gd_font}->height;
-      my $x  = ($self->{width}  - $sw) / 2;
-      my $y  = ($self->{height} - $sh) / 2;
-      $self->{image}->string($self->{gd_font}, $x, $y, $key, $color{text});
-   }
-
-   $self->$style(%color) if $self->{send_ctobg};
+   $self->insert_text($method, \%color);
+   $self->$style(%color) if     $self->{send_ctobg};
    return $self if defined wantarray;
 }
 
 # return $image_data, $image_mime_type, $random_number
 sub out {
    my $self = shift;
-   my $type = $self->{image}->can('gif') ? 'gif' : 'jpeg'; # check for older GDs
-   return $self->{image}->$type(), $type, $self->{_RANDOM_NUMBER_};
-}
-
-sub style_default {
-   my $self  = shift;
-   my $fx    = $self->{width}  / $self->{lines};
-   my $fy    = $self->{height} / $self->{lines};
-   my %color = @_;
-
-   $self->{image}->rectangle(0,0,$self->{width}-1,$self->{height}-1, $color{lines}); # put a frame around the image
-
-   for my $i (0..$self->{lines}) {
-      $self->{image}->line($i * $fx, 0,  $i * $fx     , $self->{height}, $color{lines}); # | line
-      $self->{image}->line($i * $fx, 0, ($i * $fx)+$fx, $self->{height}, $color{lines}); # \ line
-   }
-
-   for my $i (1..$self->{lines}) {
-      $self->{image}->line(0, $i * $fy, $self->{width}, $i * $fy, $color{lines}); # - line
+   if ($self->{IS_MAGICK}) {
+      $self->{image}->Set(magick => 'gif');
+      return $self->{image}->ImageToBlob, 'gif', $self->{_RANDOM_NUMBER_};
+   } else {
+      my $type = $self->{image}->can('gif') ? 'gif' : 'jpeg'; # check for older GDs
+      return $self->{image}->$type(), $type, $self->{_RANDOM_NUMBER_};
    }
 }
 
-sub style_rect {
-   my $self  = shift;
-   my $fx    = $self->{width}  / $self->{lines};
-   my $fy    = $self->{height} / $self->{lines};
-   my %color = @_;
-
-   $self->{image}->rectangle(0,0,$self->{width}-1,$self->{height}-1, $color{lines}); # put a frame around the image
-
-   for my $i (0..$self->{lines}) {
-      $self->{image}->line($i * $fx, 0,  $i * $fx     , $self->{height}, $color{lines}); # | line
-   }
-
-   for my $i (1..$self->{lines}) {
-      $self->{image}->line(0, $i * $fy, $self->{width}, $i * $fy, $color{lines}); # - line
-   }
-}
-
-sub style_box {
-   my $self  = shift;
-   my %color = @_;
-   my $w = $self->{lines};
-   $self->{image}->filledRectangle(0 , 0 , $self->{width}         , $self->{height}         , $color{text});
-   $self->{image}->filledRectangle($w, $w, $self->{width} - $w - 1, $self->{height} - $w - 1, $color{lines} );
-}
-
-
-sub style_circle {
-   my $self  = shift;
-   my $cx    = $self->{width}  / 2;
-   my $cy    = $self->{height} / 2;
-   my %color = @_;
-   my $max   = int $self->{width} / $self->{lines};
-      $max++;
-
-   for(1..$self->{lines}){
-      $self->{image}->arc($cx,$cy,$max*$_,$max*$_,0,360,$color{lines});
-   }
-}
-
-sub style_ellipse {
-   my $self  = shift;
-   my $cx    = $self->{width}  / 2;
-   my $cy    = $self->{height} / 2;
-   my %color = @_;
-   my $max   = int $self->{width} / $self->{lines};
-      $max++;
-
-   for(1..$self->{lines}){
-      $self->{image}->ellipse($cx,$cy,$max*$_*2,$max*$_,$color{lines});
-   }
-}
-
-#$image->ellipse($cx,$cy,$width,$height,$color)
-
+sub raw {shift->{image}} # raw image object
 
 1;
 
@@ -223,6 +147,45 @@ or you can just say
 to create a security image with the default settings. But that may not be 
 usefull.
 
+If you C<require> the module, you B<must> import it also:
+
+   require GD::SecurityImage;
+   import GD::SecurityImage;
+
+or:
+
+   require GD::SecurityImage;
+   GD::SecurityImage->import;
+
+if you don't like indirect object syntax.
+
+If you dont C<import>, the required modules will not be loaded and probably, 
+you'll C<die()>.
+
+Beginning with v1.2, the module supports C<Image::Magick>, but the default
+interface uses C<GD> module. To enable C<Image::Magick> support, you must 
+call the module with the C<use_magick> option:
+
+   use GD::SecurityImage use_magick => 1;
+
+If you C<require> the module, you B<must> import it also:
+
+   require GD::SecurityImage;
+   import GD::SecurityImage use_magick => 1;
+
+or:
+
+   require GD::SecurityImage;
+   GD::SecurityImage->import(use_magick => 1);
+
+if you don't like indirect object syntax.
+
+If you dont C<import>, the required modules will not be loaded and probably, 
+you'll C<die()>.
+
+The module does not I<export> anything actually. But C<import> loads the 
+necessary sub modules.
+
 =head1 DESCRIPTION
 
 The (so called) I<"Security Images"> are so popular. Most internet 
@@ -316,7 +279,8 @@ Do not use with the C<box> style.
 
 Creates the random security string or sets the random string to 
 the value you have passed. If you pass your own random string, be aware 
-that it must be at least six characters long.
+that it must be at least six (defined by a class variable) characters 
+long.
 
 =head2 random_str
 
@@ -330,9 +294,45 @@ none are mandatory.
    $image->create($method, $style, $text_color, $line_color);
 
 C<$method> can be C<normal> or C<ttf>.
-C<$style> can be C<default> or C<rect> or C<box> or C<circle> or C<ellipse>.
-The last two arguments are the colors used in the image and they are 
-passed as a 3-element (red, green and blue) arrayref.
+
+C<$style> can be one of the following:
+
+=over 4
+
+=item default
+
+The default style. Draws horizontal, vertical and angular lines.
+
+=item rect
+
+Draws horizontal and vertical lines
+
+=item box
+
+Draws two filled rectangles.
+
+The C<lines> option passed to L<new|/new>, controls the size of the inner rectangle
+for this style. If you increase the C<lines>, you'll get a smaller internal 
+rectangle. Using smaller values like C<5> can be better.
+
+=item circle
+
+Draws circles.
+
+=item ellipse
+
+Drawas ellipses.
+
+=item ec
+
+This is the combination of ellipse and circle styles. Draws both ellipses
+and circles.
+
+=back
+
+The last two arguments are the colors used in the image 
+(text and line color -- respectively) and they are passed 
+as a 3-element (red, green and blue) arrayref.
 
    $image->create($method, $style, [0,0,0], [200,200,200]);
 
@@ -344,9 +344,24 @@ C<gif> types, while new versions support C<jpeg> and C<png>.
 
 The returned mime type is either C<gif> or C<jpeg>.
 
+=head2 raw
+
+Returns the raw C<GD::Image> object:
+
+   my $i = $image->raw;
+   print $i->png;
+
+or the raw C<Image::Magick> object:
+
+   my $i = $image->raw;
+   $i->Write("gif:-");
+
+Can be usefull, if you want to modify the graphic yourself, or want to
+use another output format like C<png>.
+
 =head1 SEE ALSO
 
-L<GD>.
+L<GD>, L<ImagePwd>.
 
 =head1 EXAMPLES
 
@@ -394,8 +409,6 @@ L<GD>.
                   height   => 35,
                   ptsize   => 15,
                   lines    => 10,
-                  rndmax   => 6,
-                  rnd_data => [0..9, 'A'..'Z'],
                   gd_font  => 'giant',
                   bgcolor  => [115, 255, 255],
    );
@@ -408,15 +421,72 @@ L<GD>.
    print $cgi->header(-type => "image/$mime_type");
    print $image_data;
 
+=head2 Image::Magick example
+
+   #!/usr/bin/perl -w
+   use strict;
+   use CGI;
+   use GD::SecurityImage use_magick => 1;
+
+   my $cgi = CGI->new;
+
+   my $ttf = "/absolute/path/to/your.ttf";
+
+   my $image = GD::SecurityImage->new(
+                  width    => 90,
+                  height   => 35,
+                  ptsize   => 15,
+                  lines    => 10,
+                  font     => $ttf,
+                  bgcolor  => [115, 255, 255],
+   );
+
+   $image->random('BLAH');
+   $image->create(ttf => 'ec', [10,10,10], [210,210,50]);
+
+   my($image_data, $mime_type, $random_string) = $image->out;
+
+   print $cgi->header(-type => "image/$mime_type");
+   print $image_data;
+
+=head2 require example
+
+   #!/usr/bin/perl -w
+   use strict;
+   use CGI;
+   require GD::SecurityImage;
+   import  GD::SecurityImage use_magick => 1;
+
+   my $cgi = CGI->new;
+
+   my $ttf = "/absolute/path/to/your.ttf";
+
+   my $image = GD::SecurityImage->new(
+                  width    => 90,
+                  height   => 35,
+                  ptsize   => 15,
+                  lines    => 10,
+                  font     => $ttf,
+                  bgcolor  => [115, 255, 255],
+   );
+
+   $image->random('FOOBAR');
+   $image->create(ttf => 'ec', [10,10,10], [210,210,50]);
+
+   my($image_data, $mime_type, $random_string) = $image->out;
+
+   print $cgi->header(-type => "image/$mime_type");
+   print $image_data;
+
 =head1 ERROR HANDLING
 
-Currently, the module does not check the return values of GD's methods.
-So, if an error occurs, you can just get an empty image instead of 
-die()ing.
+Currently, the module does not check the return values of C<GD>'s and
+C<Image::Magick>' s methods. So, if an error occurs, you may just get 
+an empty image instead of die()ing.
 
 =head1 BUGS
 
-Contact the author if you find any.
+Contact the author if you find any. You can also send requests.
 
 =head1 AUTHOR
 
